@@ -50,6 +50,11 @@
 #include "q3maps.h"
 #include "utmaps.h"
 
+/* in pref.c */
+extern void q1_cmd_or_dir_changed(struct game* g);
+extern void qw_cmd_or_dir_changed(struct game* g);
+extern void q2_cmd_or_dir_changed(struct game* g);
+
 static struct player *poqs_parse_player(char *tokens[], int num, struct server *s);
 static struct player *qw_parse_player(char *tokens[], int num, struct server *s);
 static struct player *q2_parse_player(char *tokens[], int num, struct server *s);
@@ -92,6 +97,8 @@ static int bf1942_exec (const struct condef *con, int forkit);
 static int exec_generic (const struct condef *con, int forkit);
 static int ssam_exec (const struct condef *con, int forkit);
 static int savage_exec (const struct condef *con, int forkit);
+static int netpanzer_exec (const struct condef *con, int forkit);
+static int descent3_exec (const struct condef *con, int forkit);
 
 /*
 static GList *q1_custom_cfgs (struct game* this, char *dir, char *game);
@@ -112,6 +119,7 @@ static gboolean quake_has_map(struct server* s);
 static void q3_init_maps(enum server_type);
 static size_t q3_get_mapshot(struct server* s, guchar** buf);
 
+static void doom3_cmd_or_dir_changed(struct game* g);
 static void doom3_init_maps(enum server_type);
 static size_t doom3_get_mapshot(struct server* s, guchar** buf);
 static gboolean doom3_has_map(struct server* s);
@@ -146,6 +154,8 @@ static struct quake_private cod_private;
 static struct quake_private jk3_private;
 static struct quake_private doom3_private;
 
+static void ut2004_cmd_or_dir_changed(struct game* g);
+
 #include "games.c"
 
 struct gsname2type_s
@@ -439,6 +449,8 @@ static struct player *descent3_parse_player (char *token[], int n, struct server
 
   player->name = (char *) player + sizeof (struct player);
   strcpy (player->name, token[0]);
+  if(token[0][0] == '-' && token[0][strlen(token[0])-1] == '-')
+    ++s->curbots;
 
   return player;
 }
@@ -621,6 +633,9 @@ static void un_analyze_serverinfo (struct server *s) {
     else if (strcmp (*info_ptr, "hostport") == 0) {
       hostport = atoi(info_ptr[1]);
     }
+    else if (strcmp (*info_ptr, "Mutator") == 0 && strstr(info_ptr[1], "AntiTCC")) {
+      s->flags |= SERVER_PUNKBUSTER;
+    }
     else if ((s->type == UT2004_SERVER || s->type == UT2_SERVER)
 	&& strcmp (*info_ptr, "ServerVersion") == 0
 	&& strlen(info_ptr[1]) == 4)
@@ -1535,6 +1550,14 @@ static void doom3_analyze_serverinfo (struct server *s) {
       if ((n & 2) != 0)
 	s->flags |= SERVER_SP_PASSWORD;
     }
+    else if (strcmp (*info_ptr, "osmask") == 0) {
+      char* p;
+      unsigned long n = strtol(info_ptr[1], &p, 0);
+      if(info_ptr[1] != p && (n & 4) != 4)
+      {
+	s->flags |= SERVER_INCOMPATIBLE;
+      }
+    }
   }
 
   if(fs_game)
@@ -2788,6 +2811,33 @@ static int ssam_exec(const struct condef *con, int forkit) {
   return retval;
 }
 
+// Netpanzer: only supports -c
+static int netpanzer_exec(const struct condef *con, int forkit) {
+  char *argv[32];
+  int argi = 0;
+  char *cmd;
+  struct game *g = &games[con->s->type];
+  int retval;
+
+  cmd = strdup_strip (g->cmd);
+
+  argv[argi++] = strtok (cmd, delim);
+  while ((argv[argi] = strtok (NULL, delim)) != NULL)
+    argi++;
+
+  if (con->server) {
+    argv[argi++] = "-c";
+    argv[argi++] = con->server;
+  }
+
+  argv[argi] = NULL;
+
+  retval = client_launch_exec (forkit, g->real_dir, argv, con->s);
+
+  g_free (cmd);
+  return retval;
+}
+
 // launch any game that uses the gamespy protocol
 // the first argument is the content of gamename field (may be empty),
 // the second one the ip of the server
@@ -2798,7 +2848,7 @@ static int gamespy_exec (const struct condef *con, int forkit) {
   struct game *g = NULL;
   int retval;
   char **info_ptr;
-  char* gamename="";
+  char* gamename=NULL;
 
   char* hostport=NULL;
   char* real_server=NULL;
@@ -2817,14 +2867,17 @@ static int gamespy_exec (const struct condef *con, int forkit) {
   // go through all server rules
   for (info_ptr = con->s->info; info_ptr && *info_ptr; info_ptr += 2) {
     if (!strcmp (*info_ptr, "gamename")) {
-      gamename=info_ptr[1];
+      gamename = strdup_strip(info_ptr[1]);
     }
     else if (!strcmp (*info_ptr, "hostport")) {
       hostport=info_ptr[1];
     }
   }
 
-  argv[argi++] = strdup_strip (gamename);
+  if(gamename)
+    argv[argi++] = gamename;
+  else
+    argv[argi++] = "";
   
   if (con->server) {
     // gamespy port can be different from game port
@@ -2845,6 +2898,7 @@ static int gamespy_exec (const struct condef *con, int forkit) {
 
   g_free (cmd);
   g_free (real_server);
+  g_free (gamename);
   return retval;
 }
 
@@ -2948,6 +3002,59 @@ static int bf1942_exec (const struct condef *con, int forkit) {
   return retval;
 }
 
+static int descent3_exec (const struct condef *con, int forkit) {
+  char *argv[32];
+  int argi = 0;
+  char *cmd;
+  struct game *g = NULL;
+  int retval;
+  char **info_ptr;
+
+  char* hostport=NULL;
+  char* real_server=NULL;
+  
+  if(!con || !con->s)
+    return 1;
+
+  g = &games[con->s->type];
+
+  cmd = strdup_strip (g->cmd);
+
+  argv[argi++] = strtok (cmd, delim);
+  while ((argv[argi] = strtok (NULL, delim)) != NULL)
+    argi++;
+
+  // go through all server rules
+  for (info_ptr = con->s->info; info_ptr && *info_ptr; info_ptr += 2) {
+    if (!strcmp (*info_ptr, "hostport")) {
+      hostport=info_ptr[1];
+    }
+  }
+
+  if (con->server) {
+    argv[argi++] = "--directip";
+    // gamespy port can be different from game port
+    if(hostport)
+    {
+      real_server = g_strdup_printf ("%s:%s", inet_ntoa (con->s->host->ip), hostport);
+      argv[argi++] = real_server;
+    }
+    else
+    {
+      argv[argi++] = con->server;
+    }
+  }
+
+  argv[argi] = NULL;
+
+  retval = client_launch_exec (forkit, g->real_dir, argv, con->s);
+
+  g_free (cmd);
+  g_free (real_server);
+  return retval;
+}
+
+
 static char *dir_custom_cfg_filter (const char *dir, const char *str) {
   static const char *cfgext[] = { ".cfg", ".scr", ".rc", NULL };
   const char **ext;
@@ -3169,6 +3276,7 @@ static void quake_save_info (FILE *f, struct server *s) {
 
       case UN_SERVER:
       case UT2_SERVER:
+      case UT2004_SERVER:
       case RUNE_SERVER:
       case POSTAL2_SERVER:
       case AAO_SERVER:
@@ -3205,6 +3313,19 @@ static void quake_save_info (FILE *f, struct server *s) {
 		 (p->model)? p->model : ""); // player_type
 	break;
 
+      case DESCENT3_SERVER:
+	fprintf (f, 
+		 "%s" QSTAT_DELIM_STR 
+		 "%d" QSTAT_DELIM_STR 
+		 "%d" QSTAT_DELIM_STR 
+		 "%d" QSTAT_DELIM_STR
+		 "\n",
+		 p->name?p->name:"",
+		 p->frags,
+		 0, // deaths
+		 p->ping);
+	break;
+
       default:
 	fprintf (f, 
 		 "%s" QSTAT_DELIM_STR 
@@ -3426,4 +3547,74 @@ static gboolean unreal_has_map(struct server* s)
   return ut_lookup_map(hash,s->map);
 }
 
+static void doom3_cmd_or_dir_changed(struct game* g)
+{
+  FILE* f;
+  char* verinfo;
+  const char* attrproto;
+  char line[64];
+
+  debug(3, "cmd: %s, dir: %s", g->cmd, g->real_dir);
+
+  attrproto = game_get_attribute(g->type,"masterprotocol");
+
+  verinfo = file_in_dir (g->real_dir, "version.info");
+  f = fopen(verinfo, "r");
+  if(!f)
+    goto out;
+
+  if(!fgets(line, sizeof(line), f))
+    goto out;
+
+  if(!fgets(line, sizeof(line), f))
+    goto out;
+
+  debug(3, "detected doom3 protocol version %s", line);
+
+  game_set_attribute(g->type, "_masterprotocol", g_strdup(line));
+
+out:
+  if(f) fclose(f);
+  g_free(verinfo);
+}
+
+static void ut2004_cmd_or_dir_changed(struct game* g)
+{
+  FILE* f;
+  char* keyfile = NULL;
+
+  debug(2, "cmd: %s, dir: %s, home: %s", g->cmd, g->real_dir, g->real_home);
+
+  keyfile = file_in_dir (g->real_home, "System/cdkey");
+  if(keyfile)
+  {
+    f = fopen(keyfile, "r");
+    if(!f)
+    {
+      g_free(keyfile);
+      keyfile = NULL;
+    }
+  }
+
+  if(!keyfile)
+  {
+    if(f) fclose(f);
+    keyfile = file_in_dir (g->real_dir, "System/cdkey");
+    if(keyfile)
+    {
+      f = fopen(keyfile, "r");
+      if(!f)
+      {
+	g_free(keyfile);
+	keyfile = NULL;
+      }
+    }
+  }
+
+  if(keyfile || (!keyfile && game_get_attribute(g->type,"cdkey")))
+    game_set_attribute(g->type, "cdkey", keyfile);
+
+  if(f) fclose(f);
+}
+
 // vim: sw=2
