@@ -243,7 +243,7 @@ static int server_clist_refresh_row (struct server *s, int row) {
                    (s->name)? s->name : "", 2, server_pixmap, server_pixmask);
 
 #ifdef USE_GEOIP
-  countrypix = get_pixmap_for_country(s->country_id);
+  countrypix = get_pixmap_for_country_with_fallback(s->country_id);
   if(countrypix)
 	  gtk_clist_set_pixtext (server_clist, row, 1, 
                    text[1], 2, countrypix->pix, countrypix->mask);
@@ -252,7 +252,11 @@ static int server_clist_refresh_row (struct server *s, int row) {
   // if map not available
   if(games[s->type].has_map && games[s->type].has_map(s) == FALSE)
   {
-    /*
+    gtk_clist_set_pixtext (server_clist, row, 6, s->map, 2, rminus_pix.pix, rminus_pix.mask );
+  }
+
+  if(s->flags & SERVER_INCOMPATIBLE)
+  {
     GtkStyle *style;
 
     style = gtk_widget_get_style(GTK_WIDGET(server_clist));
@@ -260,8 +264,6 @@ static int server_clist_refresh_row (struct server *s, int row) {
     {
       gtk_clist_set_foreground(server_clist,row,&style->fg[GTK_STATE_INSENSITIVE]);
     }
-    */
-    gtk_clist_set_pixtext (server_clist, row, 6, s->map, 2, rminus_pix.pix, rminus_pix.mask );
   }
 
   gdk_pixmap_unref (server_pixmap);
