@@ -73,17 +73,20 @@
 #define	T2M_DEFAULT_PORT	28000		/* Tribes 2 */
 #define	SOF2M_DEFAULT_PORT	20110		/* Soldier of Fortune 2 */
 
-#define PLAYER_GROUP_MASK	0x07
-#define PLAYER_GROUP_RED	0x01
-#define PLAYER_GROUP_GREEN	0x02
-#define PLAYER_GROUP_BLUE	0x04
+/* max 0x8000, server->flags is unsigned */
+enum server_flags {
+  PLAYER_GROUP_RED =	0x001,
+  PLAYER_GROUP_GREEN =	0x002,
+  PLAYER_GROUP_BLUE =	0x004,
+  PLAYER_GROUP_MASK =	0x007,
 
-/* max 0x80, server->flags is char */
-#define SERVER_CHEATS		0x08
-#define SERVER_PASSWORD		0x10
-#define SERVER_SP_PASSWORD	0x20
-#define SERVER_SPECTATE		0x40
-#define SERVER_PUNKBUSTER	0x80
+  SERVER_CHEATS =	0x008,
+  SERVER_PASSWORD =	0x010,
+  SERVER_SP_PASSWORD =	0x020,
+  SERVER_SPECTATE =	0x040,
+  SERVER_PUNKBUSTER =	0x080,
+  SERVER_INCOMPATIBLE =	0x100
+};
 
 enum launch_mode { 
   LAUNCH_NORMAL,
@@ -124,8 +127,9 @@ enum server_type {
   COD_SERVER,
   SAS_SERVER,
   BF1942_SERVER,
-  GPS_SERVER,
   JK3_SERVER,
+  NETP_SERVER,
+  GPS_SERVER,
   UNKNOWN_SERVER
 };
 
@@ -145,6 +149,7 @@ enum master_query_type {
 	MASTER_HTTP,
 	MASTER_LAN,
 	MASTER_FILE,
+	MASTER_GSLIST,
 	MASTER_NUM_QUERY_TYPES,
 	MASTER_INVALID_TYPE
 };
@@ -201,7 +206,7 @@ struct server {
   int country_id;
 #endif
 
-  unsigned char flags;
+  unsigned flags;
 
   unsigned char filters;
   unsigned char flt_mask;
@@ -221,11 +226,17 @@ struct userver {
   int 	ref_count;
 };
 
-struct master {
+typedef struct {
+  int portadjust;      /* value added to port */
+  char* gsmtype;        /* type for gslist master */
+} QFMasterOptions;
+
+typedef struct master {
   char *name;
   enum server_type type;
   int isgroup;		/* is it a real master or master group? */
   int user;		/* is it added or edited by user? */
+  QFMasterOptions options;
 
   struct host *host;
   unsigned short port;
@@ -243,11 +254,13 @@ struct master {
 
   /** private */
   char* _qstat_master_option; // optional override from games[type].qstat_master_option
-};
+} QFMaster;
 
 char* master_qstat_option(struct master* m);
 void master_set_qstat_option(struct master* m, const char* opt);
 
+char* master_to_url(QFMaster* m);
+
 extern	time_t xqf_start_time;
 extern char* xqf_PACKAGE_DATA_DIR;
 extern char* xqf_LOCALEDIR;
